/* dcast_consequence.c
   dreamcast asteroids v0.0.3: split screen

 by
 jimbo
 Dan Potter and Jordan DeLong of Cryptic Allusion
 Andrew K.
 Marcus Comstedt

 this file contains the rules of the game.
  what happens when shots hit (and how to tell)
*/





/* vector dot and cross product in 3D only for intersection calculation
 *
 */
inline double dot_product( double *a, double *b ) {
  return a[0] * b[0]  + a[1] * b[1]  + a[2] * b[2];
}
inline void cross_product( double *a, double *b, double *c ) {
  c[0]  =  b[1] * a[2]  -  a[1] * b[2];
  c[1]  =  b[2] * a[0]  -  b[0] * a[2];
  c[2]  =  b[0] * a[1]  -  a[0] * b[1];
}

/*
 * consequences
 *
 *
 *
 */
void determine_fate( shooter_t *p ) {
    int j,k;
    float player_radius = 0.3;


    if ( p->living == 0 ) { return; }
    
    /* v0.0.6: player death
     *    remove all items
     *
     */
    if ( p->hull_integrity <= 0 ) {
        create_player_explosion( p );
        p->living = 0;
        p->machine_gun = 0;
        p->spread_gun = 0;
        return;
    }




    /*** first find direction in terms of yaw and pitch **/
    p->direction[0] = cos( p->yaw );
    p->direction[2] = sin( p->yaw );
    p->direction[1] = sin( p->pitch );

    p->direction[0] *= cos( p->pitch );
    p->direction[2] *= cos( p->pitch );





    /* wall condition here:
     *  if exceeding the space of the arena,
     *   reverse direction along that axis
     *
     *
     * v0.0.5: this could be scaled by some factor...
     * v0.0.6: fix the potential trap here - add some space between player and wall
     *
     *
     */
    for ( k=0; k<3; k++ ) {
        if ( ( p->location[k] > 2.0 * bound_size ) || ( p->location[k] < -2.0 * bound_size ) ) {

            if ( p->location[k] > 2.0 * bound_size ) {
                p->location[k] -= 1.0;
            }
            if ( p->location[k] < -2.0 * bound_size ) {
                p->location[k] += 1.0;
            }
            p->velocity[k] = -p->velocity[k];
#ifdef SOUND_EFFECTS
            snd_sfx_play( 5, 240, 1 );
#endif
            /*** v0.0.6: damage ***/
            p->hull_integrity--;

        }
    }





    /*** animation : move player forward by direction * timestep ***/
    p->location[0] += p->velocity[0];
    p->location[1] += p->velocity[1];
    p->location[2] += p->velocity[2];

    /*
     *
     * test for collisions with player... could move into other loop
     *
     *
     */
    for ( j=0; j<asteroid_count; j++ ) {
        float sum = 0.0;

        sum += ( p->location[0] - asteroids[j]->location[0] ) * ( p->location[0] - asteroids[j]->location[0] );
        sum += ( p->location[1] - asteroids[j]->location[1] ) * ( p->location[1] - asteroids[j]->location[1] );
        sum += ( p->location[2] - asteroids[j]->location[2] ) * ( p->location[2] - asteroids[j]->location[2] );
        sum = sqrt(sum);

        if ( sum < asteroids[j]->radius + player_radius ) {
                    
            //    p->damage_hits += asteroids[j]->size;
            p->hull_integrity -= 10 * asteroids[j]->size;


            p->velocity[0] += (float)asteroids[j]->size * asteroids[j]->velocity[0];
            p->velocity[1] += (float)asteroids[j]->size * asteroids[j]->velocity[1];
            p->velocity[2] += (float)asteroids[j]->size * asteroids[j]->velocity[2];
                    
                    
#if 0
            /*** remove the asteroid? ***/
            /*** smallest asteroid - destroy it - shuffle down ***/
            if ( asteroids[j]->size >= 1  ) {
                int k;
                free( asteroids[j] );
                for ( k=j; k<asteroid_count; k++ ) {
                    asteroids[k] = asteroids[k+1];
                }
                asteroid_count--;
            }
#endif
                    

        }




    }
}



/* motion
 *
 *
 *
 */
void move_asteroids() {
    int j;

    for ( j=0; j<asteroid_count; j++ ) {
        int k;
        asteroids[j]->location[0] += asteroids[j]->velocity[0];
        asteroids[j]->location[1] += asteroids[j]->velocity[1];
        asteroids[j]->location[2] += asteroids[j]->velocity[2];

        
        asteroids[j]->rotation[0] += asteroids[j]->rotation_speed[0];
        asteroids[j]->rotation[1] += asteroids[j]->rotation_speed[1];
        asteroids[j]->rotation[2] += asteroids[j]->rotation_speed[2];


        /* wall condition here:
         *  if exceeding the space of the arena,
         *   reverse direction along that axis
         */
        for ( k=0; k<3; k++ ) {
            if ( asteroids[j]->location[k] > bound_size ) {
                asteroids[j]->velocity[k] = -asteroids[j]->velocity[k];
            }
            if ( asteroids[j]->location[k] < -bound_size ) {
                asteroids[j]->velocity[k] = -asteroids[j]->velocity[k];
            }
        }


    }



    /* move shots
     *
     *
     */
    for ( j=0; j<active_moving_shot_count; j++ ) {
        active_moving_shots[j]->location[0] += active_moving_shots[j]->direction[0];
        active_moving_shots[j]->location[1] += active_moving_shots[j]->direction[1];
        active_moving_shots[j]->location[2] += active_moving_shots[j]->direction[2];
        active_moving_shots[j]->lifetime++;

        /*** destroy after lifetime ***/
        if ( active_moving_shots[j]->lifetime > 60 ) {
            int k;
            free( active_moving_shots[j] );
            for ( k=j; k<active_moving_shot_count; k++ ) {
                active_moving_shots[k] = active_moving_shots[k+1];
            }
            active_moving_shot_count--;
        }

    }



    /* v0.0.5: update explosions
     *
     *
     *
     */
    for ( j=0; j<explosion_count; j++ ) {
        explosions[j]->lifetime++;

        /*** destroy after lifetime ***/
        if ( explosions[j]->lifetime > MAX_EXPLOSION_LIFETIME ) {
            int k;
            free( explosions[j] );
            for ( k=j; k<explosion_count; k++ ) {
                explosions[k] = explosions[k+1];
            }
            explosion_count--;
        }

    }


    /* v0.0.6: move enemies
     *
     *
     *  AI code goes here
     *
     *
     *
     */
    for ( j=0; j<enemy_count; j++ ) {
        double sum = 0.0;
        double accel[4] = { 0.0, 0.0, 0.0, 0.0} ;
        
        enemies[j]->lifetime++;

        /*** check if dead ***/
        //    if ( enemies[j]->hull_integrity <= 0 ) {
        if ( ( enemies[j]->hull_integrity <= 0 ) || ( enemies[j]->lifetime > (enemy_giveup + ENEMY_FLYAWAYTIME) ) ) {
            int k;


            /*** make big explosion ***/
            if ( enemies[j]->hull_integrity <= 0 )
                create_enemy_explosion( enemies[j] );
            
#ifdef SOUND_EFFECTS
            snd_sfx_play( 6, 240, 1 );
#endif


            if ( enemies[j]->opponent != NULL )
                enemies[j]->opponent->score += 100;



            free( enemies[j] );
            for ( k=j; k<enemy_count; k++ ) {
                enemies[k] = enemies[k+1];
            }
            enemy_count--;
        }
        /*** live enemy ***/
        else {
            float minimum_distance = 15.0;
            double up[4] = {0.0, 1.0, 0.0, 0.0 };
            double perp_vector[4];




            /*** give up after some time ***/
            if ( enemies[j]->lifetime > enemy_giveup ) {
                enemies[j]->location[1] += 0.2;
            }
            else {




                // dead opponent - find first living guy
                if ( ( enemies[j]->opponent == NULL ) || ( enemies[j]->opponent->living == 0 ) ) {
                    int i=0;
                    while ( ( players[i]->living != 1 ) && ( i < player_count ) ) { i++; }
                    if ( i < player_count )
                        enemies[j]->opponent = players[i];
                    else enemies[j]->opponent = NULL;
                }

                // accel[2] += players[0]->location[2] - enemies[j]->location[2];

                accel[0] += enemies[j]->opponent->location[0] - enemies[j]->location[0];
                accel[1] += enemies[j]->opponent->location[1] - enemies[j]->location[1];
                accel[2] += enemies[j]->opponent->location[2] - enemies[j]->location[2];

                sum += accel[0] * accel[0];
                sum += accel[1] * accel[1];
                sum += accel[2] * accel[2];
                sum = sqrt( sum );
        
                /*** within range ***/
                if ( sum > minimum_distance ) {


                    sum *= 10.0;
        

                    enemies[j]->velocity[0] += accel[0] / sum;
                    enemies[j]->velocity[1] += accel[1] / sum;
                    enemies[j]->velocity[2] += accel[2] / sum;

                    // normalize velocity
                    sum = 0.0;
                    sum += enemies[j]->velocity[0] * enemies[j]->velocity[0];
                    sum += enemies[j]->velocity[1] * enemies[j]->velocity[1];
                    sum += enemies[j]->velocity[2] * enemies[j]->velocity[2];
        
                    sum = sqrt( sum );

                    // sum *= 10.0;
                    sum *= 4.0;
                    enemies[j]->velocity[0] /= sum;
                    enemies[j]->velocity[1] /= sum;
                    enemies[j]->velocity[2] /= sum;

                    enemies[j]->location[0] += enemies[j]->velocity[0];
                    enemies[j]->location[1] += enemies[j]->velocity[1];
                    enemies[j]->location[2] += enemies[j]->velocity[2];
                }
#if 0
                /*** cross product - swing around ***/
                else {
                    float cross_scale = 1.0;
                    cross_product( up, enemies[j]->direction, perp_vector );
                    enemies[j]->velocity[0] += cross_scale * perp_vector[0];
                    enemies[j]->velocity[1] += cross_scale * perp_vector[1];
                    enemies[j]->velocity[2] += cross_scale * perp_vector[2];

                }
#endif

                /*** fire at player ***/
                if ( enemies[j]->shot_cooldown == 0 ) {
                    enemies[j]->shot_cooldown = 20;

                    /* add a shot to the scene
                     *  
                     *
                     */
                    if ( active_moving_shot_count < MAX_SHOTS ) {
                        double shot_boost = 3.0;
                        moving_shot_t *new_shot = malloc( sizeof( moving_shot_t) );

                        new_shot->location[0] = enemies[j]->location[0];
                        new_shot->location[1] = enemies[j]->location[1];
                        new_shot->location[2] = enemies[j]->location[2];
#if 0
                        new_shot->direction[0] = shot_boost * enemies[j]->direction[0];
                        new_shot->direction[1] = shot_boost * enemies[j]->direction[1];
                        new_shot->direction[2] = shot_boost * enemies[j]->direction[2];
#endif 
                        new_shot->direction[0] = shot_boost * enemies[j]->velocity[0];
                        new_shot->direction[1] = shot_boost * enemies[j]->velocity[1];
                        new_shot->direction[2] = shot_boost * enemies[j]->velocity[2];

                        /*** add ship velocity for truer shot ***/
                        new_shot->direction[0] += enemies[j]->velocity[0];
                        new_shot->direction[1] += enemies[j]->velocity[1];
                        new_shot->direction[2] += enemies[j]->velocity[2];

                        new_shot->lifetime = 0;
                        new_shot->owner = NULL;

                        active_moving_shots[ active_moving_shot_count ] = new_shot;
                        active_moving_shot_count++;
                    }
                }
                else {
                    enemies[j]->shot_cooldown--;
                }
            }


        }




    }


    /* v0.0.6: move powerups
     *
     *
     */
    for ( j=0; j<powerup_count; j++ ) {
        int k;
        powerups[j]->location[0] += powerups[j]->velocity[0];
        powerups[j]->location[1] += powerups[j]->velocity[1];
        powerups[j]->location[2] += powerups[j]->velocity[2];

        
        powerups[j]->rotation[0] += powerups[j]->rotation_speed[0];
        powerups[j]->rotation[1] += powerups[j]->rotation_speed[1];
        powerups[j]->rotation[2] += powerups[j]->rotation_speed[2];


        /* wall condition here:
         *  if exceeding the space of the arena,
         *   reverse direction along that axis
         */
        for ( k=0; k<3; k++ ) {
            if ( powerups[j]->location[k] > bound_size ) {
                powerups[j]->velocity[k] = -powerups[j]->velocity[k];
            }
            if ( powerups[j]->location[k] < -bound_size ) {
                powerups[j]->velocity[k] = -powerups[j]->velocity[k];
            }
        }


    }



}



/* trouble when moving this into a function - fpu error
 *
 *
 *
 *
 */
void process_asteroid_hit( int i, int j ) {
    int i;
    double rad = asteroids[j]->radius;


    asteroids[j]->location[0] += active_shots[i]->direction[0];
    asteroids[j]->location[1] += active_shots[i]->direction[1];
    asteroids[j]->location[2] += active_shots[i]->direction[2];


    /*** smallest asteroid - destroy it - shuffle down ***/
    if ( asteroids[j]->size <= 1  ) {
        int k;
        free( asteroids[j] );
        for ( k=j; k<asteroid_count; k++ ) {
            asteroids[k] = asteroids[k+1];
        }
        asteroid_count--;
    }

    /* big asteroid:
     *  create a new one and add it 
     *   also downsize old one to be the other half
     *
     */
    else if ( asteroid_count < MAX_ASTEROIDS ) {
        float blast_vector[4];


        blast_vector[0] = (float)(rand()%100 - 50)/500.0f;
        blast_vector[1] = (float)(rand()%100 - 50)/500.0f;
        blast_vector[2] = (float)(rand()%100 - 50)/500.0f;

        asteroids[ asteroid_count ] = malloc( sizeof( asteroid_t ) );
        init_asteroid( asteroids[ asteroid_count ] );

        /*** old asteroid - downsize, add velocity from blast ***/
        asteroids[j]->size--;
        asteroids[j]->radius = (float)asteroids[j]->size;
        asteroids[j]->velocity[0] += blast_vector[0];
        asteroids[j]->velocity[1] += blast_vector[1];
        asteroids[j]->velocity[2] += blast_vector[2];
                    

        asteroids[ asteroid_count ]->location[0] = asteroids[j]->location[0] + rad * 0.5;
        asteroids[ asteroid_count ]->location[1] = asteroids[j]->location[1] + rad * 0.5;
        asteroids[ asteroid_count ]->location[2] = asteroids[j]->location[2] + rad * 0.5;

        asteroids[ asteroid_count ]->velocity[0] -= blast_vector[0];
        asteroids[ asteroid_count ]->velocity[1] -= blast_vector[1];
        asteroids[ asteroid_count ]->velocity[2] -= blast_vector[2];

        asteroids[ asteroid_count ]->size = asteroids[j]->size;
        asteroids[ asteroid_count ]->radius = (float)asteroids[j]->size;

        asteroid_count++;

    }








}






/* process active shots:
 *   see if any of them hit anything.
 *    - if they did, take action
 *
 */
void process_shots() {
    int i;




    /*** laser shots ***/
    for ( i=0; i<active_shot_count; i++ ) {
        int j;

        /* all asteroids:
         *
         *
         */
        for ( j=0; j<asteroid_count; j++ ) {
            /*** sphere intersection code. ***/
            double a, b, c, determinant;
            int intersects = 0;
            a = dot_product( active_shots[i]->direction , active_shots[i]->direction );
            b = 2 * ( dot_product( active_shots[i]->origin, active_shots[i]->direction )
                      - dot_product( active_shots[i]->direction, asteroids[j]->location )  );
            c = dot_product( active_shots[i]->origin, active_shots[i]->origin ) 
                + dot_product( asteroids[j]->location, asteroids[j]->location ) 
                - 2 * dot_product( active_shots[i]->origin, asteroids[j]->location )
                - asteroids[j]->radius * asteroids[j]->radius;
            determinant = b*b - 4*a*c;
            if ( determinant < 0 ) { intersects = 0; }
            if ( determinant < 0.000000001 ) { intersects = 1; }
            else { intersects = 2; }
                


            /* hit
             *
             *
             */
            if ( intersects > 1 ) {
                double rad = asteroids[j]->radius;


                /*** scoring ***/
                active_shots[i]->owner->score += size_to_score( asteroids[j]->size );



                asteroids[j]->location[0] += active_shots[i]->direction[0];
                asteroids[j]->location[1] += active_shots[i]->direction[1];
                asteroids[j]->location[2] += active_shots[i]->direction[2];


                /*** smallest asteroid - destroy it - shuffle down ***/
                if ( asteroids[j]->size <= 1  ) {
                    int k;
                    free( asteroids[j] );
                    for ( k=j; k<asteroid_count; k++ ) {
                        asteroids[k] = asteroids[k+1];
                    }
                    asteroid_count--;
                }

                /* big asteroid:
                 *  create a new one and add it 
                 *   also downsize old one to be the other half
                 *
                 */
                else if ( asteroid_count < MAX_ASTEROIDS ) {
                    float blast_vector[4];


                    blast_vector[0] = (float)(rand()%100 - 50)/500.0f;
                    blast_vector[1] = (float)(rand()%100 - 50)/500.0f;
                    blast_vector[2] = (float)(rand()%100 - 50)/500.0f;

                    asteroids[ asteroid_count ] = malloc( sizeof( asteroid_t ) );
                    init_asteroid( asteroids[ asteroid_count ] );

                    /*** old asteroid - downsize, add velocity from blast ***/
                    asteroids[j]->size--;
                    asteroids[j]->radius = (float)asteroids[j]->size;
                    asteroids[j]->velocity[0] += blast_vector[0];
                    asteroids[j]->velocity[1] += blast_vector[1];
                    asteroids[j]->velocity[2] += blast_vector[2];
                    

                    asteroids[ asteroid_count ]->location[0] = asteroids[j]->location[0] + rad * 0.5;
                    asteroids[ asteroid_count ]->location[1] = asteroids[j]->location[1] + rad * 0.5;
                    asteroids[ asteroid_count ]->location[2] = asteroids[j]->location[2] + rad * 0.5;

                    asteroids[ asteroid_count ]->velocity[0] -= blast_vector[0];
                    asteroids[ asteroid_count ]->velocity[1] -= blast_vector[1];
                    asteroids[ asteroid_count ]->velocity[2] -= blast_vector[2];

                    asteroids[ asteroid_count ]->size = asteroids[j]->size;
                    asteroids[ asteroid_count ]->radius = (float)asteroids[j]->size;

                    asteroid_count++;

                }



            } /** hit **/


        }  /** all asteroids **/





        /* all players:
         *
         *
         */
        for ( j=0; j<player_count; j++ ) {

            if ( players[j]->living == 1 ) {

                /*** sphere intersection code. ***/
                double a, b, c, determinant;
                int intersects = 0;
                a = dot_product( active_shots[i]->direction , active_shots[i]->direction );
                b = 2 * ( dot_product( active_shots[i]->origin, active_shots[i]->direction )
                          - dot_product( active_shots[i]->direction, players[j]->location )  );
                c = dot_product( active_shots[i]->origin, active_shots[i]->origin ) 
                    + dot_product( players[j]->location, players[j]->location ) 
                    - 2 * dot_product( active_shots[i]->origin, players[j]->location )
                    - 9.0;
                //  - players[j]->radius * players[j]->radius;
                determinant = b*b - 4*a*c;
                if ( determinant < 0 ) { intersects = 0; }
                if ( determinant < 0.000000001 ) { intersects = 1; }
                else { intersects = 2; }
                


                /* laser hit
                 *
                 *
                 */
                if ( intersects > 1 ) {
                    if ( players[j] != active_shots[i]->owner ) {
                        if ( active_shots[i]->potent == MAX_SHOT_POTENCY ) {

#ifdef SOUND_EFFECTS
                            snd_sfx_play(0, 240, 1 );
#endif
                            //   players[j]->damage_hits++;
                            players[j]->hull_integrity -= 50;

                            // players[j]->score -= 30;
                            /*** score ***/
                            // if ( players[j]->score > 0 )
                            //  players[j]->score--;
                        }
                    }
                } /** hit **/


            }




        }  /** all players **/






        /* v0.0.6: all enemys:
         *
         *
         */
        for ( j=0; j<enemy_count; j++ ) {
            /*** sphere intersection code. ***/
            double a, b, c, determinant;
            int intersects = 0;
            a = dot_product( active_shots[i]->direction , active_shots[i]->direction );
            b = 2 * ( dot_product( active_shots[i]->origin, active_shots[i]->direction )
                      - dot_product( active_shots[i]->direction, enemies[j]->location )  );
            c = dot_product( active_shots[i]->origin, active_shots[i]->origin ) 
                + dot_product( enemies[j]->location, enemies[j]->location ) 
                - 2 * dot_product( active_shots[i]->origin, enemies[j]->location )
                - 9.0;
            //  - enemys[j]->radius * enemys[j]->radius;
            determinant = b*b - 4*a*c;
            if ( determinant < 0 ) { intersects = 0; }
            if ( determinant < 0.000000001 ) { intersects = 1; }
            else { intersects = 2; }
                


            /* hit
             *
             *
             */
            if ( intersects > 1 ) {
                if ( active_shots[i]->potent == MAX_SHOT_POTENCY ) {

#ifdef SOUND_EFFECTS
                    snd_sfx_play(0, 240, 1 );
#endif
                    enemies[j]->hull_integrity -= 10;
                }
            } /** hit **/



        }  /** all enemys **/



        /* v0.0.6: all powerups:
         *
         *
         */
        for ( j=0; j<powerup_count; j++ ) {
            /*** sphere intersection code. ***/
            double a, b, c, determinant;
            int intersects = 0;
            a = dot_product( active_shots[i]->direction , active_shots[i]->direction );
            b = 2 * ( dot_product( active_shots[i]->origin, active_shots[i]->direction )
                      - dot_product( active_shots[i]->direction, enemies[j]->location )  );
            c = dot_product( active_shots[i]->origin, active_shots[i]->origin ) 
                + dot_product( enemies[j]->location, enemies[j]->location ) 
                - 2 * dot_product( active_shots[i]->origin, enemies[j]->location )
                - 9.0;
            //  - powerups[j]->radius * powerups[j]->radius;
            determinant = b*b - 4*a*c;
            if ( determinant < 0 ) { intersects = 0; }
            if ( determinant < 0.000000001 ) { intersects = 1; }
            else { intersects = 2; }
                


            /* hit
             *
             *
             */
            if ( intersects > 1 ) {
                if ( active_shots[i]->potent == MAX_SHOT_POTENCY ) {
                    int k;

#ifdef SOUND_EFFECTS
                    snd_sfx_play( 12, 240, 1 );
#endif

                    free( powerups[j] );
                    for ( k=j; k<powerup_count; k++ ) {
                        powerups[k] = powerups[k+1];
                    }
                    powerup_count--;

                }
            } /** hit **/



        }  /** all powerups **/










        /*
         * if old remove shot from the list 
         *
         *
         */
        active_shots[i]->potent--;
        if ( active_shots[i]->potent < 0 ) {
            int k;
            free ( active_shots[i] );
            for ( k=i; k<active_shot_count; k++ ) {
                active_shots[k] = active_shots[k+1];
            }
            active_shot_count--;
        }
    }












    /**
     * check moving shots 
     *
     *
     */
    for ( i=0; i<active_moving_shot_count; i++ ) {
        int j;

        /* all asteroids:
         *
         *
         */
        for ( j=0; j<asteroid_count; j++ ) {
            double diff_vector[4], sum = 0.0;

            diff_vector[0] = active_moving_shots[i]->location[0] - asteroids[j]->location[0];
            diff_vector[1] = active_moving_shots[i]->location[1] - asteroids[j]->location[1];
            diff_vector[2] = active_moving_shots[i]->location[2] - asteroids[j]->location[2];

            sum += diff_vector[0] * diff_vector[0];
            sum += diff_vector[1] * diff_vector[1];
            sum += diff_vector[2] * diff_vector[2];
            sum = sqrt(sum);
            
            /* hit
             *
             */
            if ( sum < asteroids[j]->radius  ) {
                int k;
                double rad = asteroids[j]->radius;


                /*** scoring ***/
                active_moving_shots[i]->owner->score += size_to_score( asteroids[j]->size );

                /* v0.0.5: create an explosion
                 *
                 *
                 */
                create_explosion( active_moving_shots[i] );
#ifdef SOUND_EFFECTS
                snd_sfx_play(0, 240, 0);
#endif




                /*** kill the shot ***/
                free( active_moving_shots[i] );
                for ( k=i; k<active_moving_shot_count; k++ ) {
                    active_moving_shots[k] = active_moving_shots[k+1];
                }
                active_moving_shot_count--;






                /*** smallest asteroid - destroy it - shuffle down ***/
                if ( asteroids[j]->size <= 1  ) {
                    int k;
                    free( asteroids[j] );
                    for ( k=j; k<asteroid_count; k++ ) {
                        asteroids[k] = asteroids[k+1];
                    }
                    asteroid_count--;
                }






                /* big asteroid:
                 *  create a new one and add it 
                 *   also downsize old one to be the other half
                 *
                 */
                else if ( asteroid_count < MAX_ASTEROIDS ) {
                    float blast_vector[4];


                    blast_vector[0] = (float)(rand()%100 - 50)/500.0f;
                    blast_vector[1] = (float)(rand()%100 - 50)/500.0f;
                    blast_vector[2] = (float)(rand()%100 - 50)/500.0f;


                    /*** old asteroid - downsize, add velocity from blast ***/
                    asteroids[j]->size--;
                    asteroids[j]->radius = (float)asteroids[j]->size;
                    asteroids[j]->velocity[0] += blast_vector[0];
                    asteroids[j]->velocity[1] += blast_vector[1];
                    asteroids[j]->velocity[2] += blast_vector[2];
                    
                    /*** new asteroid ***/
                    asteroids[ asteroid_count ] = malloc( sizeof( asteroid_t ) );
                    init_asteroid( asteroids[ asteroid_count ] );

                    asteroids[ asteroid_count ]->location[0] = asteroids[j]->location[0] + rad * 0.5;
                    asteroids[ asteroid_count ]->location[1] = asteroids[j]->location[1] + rad * 0.5;
                    asteroids[ asteroid_count ]->location[2] = asteroids[j]->location[2] + rad * 0.5;

                    asteroids[ asteroid_count ]->velocity[0] -= blast_vector[0];
                    asteroids[ asteroid_count ]->velocity[1] -= blast_vector[1];
                    asteroids[ asteroid_count ]->velocity[2] -= blast_vector[2];

                    asteroids[ asteroid_count ]->size = asteroids[j]->size;
                    asteroids[ asteroid_count ]->radius = (float)asteroids[j]->size;

                    asteroid_count++;

                }


            } /** hit **/



        } /** for(asteroids) **/





        /* players
         *
         *
         *
         */
        for ( j=0; j<player_count; j++ ) {

            if ( players[j]->living == 1 ) {
                if ( players[j] != active_moving_shots[i]->owner ) {
                    double diff_vector[4], sum = 0.0;
                
                    diff_vector[0] = active_moving_shots[i]->location[0] - players[j]->location[0];
                    diff_vector[1] = active_moving_shots[i]->location[1] - players[j]->location[1];
                    diff_vector[2] = active_moving_shots[i]->location[2] - players[j]->location[2];

                    sum += diff_vector[0] * diff_vector[0];
                    sum += diff_vector[1] * diff_vector[1];
                    sum += diff_vector[2] * diff_vector[2];
                    sum = sqrt(sum);
            
                    /* hit - whats a player's radius?
                     *
                     */
                    if ( sum < 3.0 ) { //players[j]->radius  ) {
                        int k;
                        double rad = 3.0;//players[j]->radius;
                        double massratio = 0.02;
                        /* scoring - take away a point?
                         *
                         */
                        //    if ( players[j]->score > 0 )
                        players[j]->score -= 10;

#ifdef SOUND_EFFECTS
                        snd_sfx_play(0, 240, 1 );
#endif

                        /* v0.0.6: add shots velocity (*constant)
                         *
                         *
                         */
                        players[j]->velocity[0] += massratio * active_moving_shots[i]->direction[0];
                        players[j]->velocity[1] += massratio * active_moving_shots[i]->direction[1];
                        players[j]->velocity[2] += massratio * active_moving_shots[i]->direction[2];


                        /* v0.0.5: create an explosion
                         *
                         *
                         */
                        create_explosion( active_moving_shots[i] );


                        /*** kill the shot ***/
                        free( active_moving_shots[i] );
                        for ( k=i; k<active_moving_shot_count; k++ ) {
                            active_moving_shots[k] = active_moving_shots[k+1];
                        }
                        active_moving_shot_count--;

                        //  players[j]->damage_hits++;
                        players[j]->hull_integrity -= 10;

                    } /** hit **/



                } /** !owner **/
            } /** living **/
            
        } /** for(players) **/






        /* enemies
         *
         *
         *
         */
        if ( active_moving_shots[i]->owner != NULL )
        for ( j=0; j<enemy_count; j++ ) {

            double diff_vector[4], sum = 0.0;
                
            diff_vector[0] = active_moving_shots[i]->location[0] - enemies[j]->location[0];
            diff_vector[1] = active_moving_shots[i]->location[1] - enemies[j]->location[1];
            diff_vector[2] = active_moving_shots[i]->location[2] - enemies[j]->location[2];

            sum += diff_vector[0] * diff_vector[0];
            sum += diff_vector[1] * diff_vector[1];
            sum += diff_vector[2] * diff_vector[2];
            sum = sqrt(sum);
            
            /* hit - whats a enemy's radius?
             *
             */
            if ( sum < 3.0 ) { //enemys[j]->radius  ) {
                int k;
                double rad = 3.0;//enemys[j]->radius;
                double massratio = 0.02;
#ifdef SOUND_EFFECTS
                snd_sfx_play( 0, 240, 1 );
#endif

                /* v0.0.6: add shots velocity (*constant)
                 *
                 *
                 */
                enemies[j]->velocity[0] += massratio * active_moving_shots[i]->direction[0];
                enemies[j]->velocity[1] += massratio * active_moving_shots[i]->direction[1];
                enemies[j]->velocity[2] += massratio * active_moving_shots[i]->direction[2];


                /* v0.0.5: create an explosion
                 *
                 *
                 */
                create_explosion( active_moving_shots[i] );

                enemies[j]->hull_integrity -= 10;
                enemies[j]->opponent = active_moving_shots[i]->owner;
                /*** scoring ***/
                active_moving_shots[i]->owner->score += 10;


                /*** kill the shot ***/
                free( active_moving_shots[i] );
                for ( k=i; k<active_moving_shot_count; k++ ) {
                    active_moving_shots[k] = active_moving_shots[k+1];
                }
                active_moving_shot_count--;


            } /** hit **/





        } /** for(enemys) **/




        /* enemies
         *
         *
         *
         */
        // if ( active_moving_shots[i]->owner != NULL )
        for ( j=0; j<powerup_count; j++ ) {

            double diff_vector[4], sum = 0.0;
                
            diff_vector[0] = active_moving_shots[i]->location[0] - powerups[j]->location[0];
            diff_vector[1] = active_moving_shots[i]->location[1] - powerups[j]->location[1];
            diff_vector[2] = active_moving_shots[i]->location[2] - powerups[j]->location[2];

            sum += diff_vector[0] * diff_vector[0];
            sum += diff_vector[1] * diff_vector[1];
            sum += diff_vector[2] * diff_vector[2];
            sum = sqrt(sum);
            
            /* hit - whats a powerup's radius?
             *
             */
            if ( sum < 3.0 ) { //powerups[j]->radius  ) {
                int k;
                double rad = 3.0;//powerups[j]->radius;
#ifdef SOUND_EFFECTS
                snd_sfx_play(0, 240, 1 );
#endif

                free( powerups[j] );
                for ( k=j; k<powerup_count; k++ ) {
                    powerups[k] = powerups[k+1];
                }
                powerup_count--;

                /* v0.0.5: create an explosion
                 *
                 *
                 */
                create_explosion( active_moving_shots[i] );


                /*** kill the shot ***/
                free( active_moving_shots[i] );
                for ( k=i; k<active_moving_shot_count; k++ ) {
                    active_moving_shots[k] = active_moving_shots[k+1];
                }
                active_moving_shot_count--;

                enemies[j]->hull_integrity -= 10;

            } /** hit **/





        } /** for(powerups) **/












    } /** for(moving shots) **/




    /*
     * check powerups
     *
     *
     */
    for ( i=0; i<powerup_count; i++ ) {
        int j;
        for ( j=0; j<player_count; j++ ) {

            if ( players[j]->living == 1 ) {
                    double diff_vector[4], sum = 0.0;
                
                    diff_vector[0] = powerups[i]->location[0] - players[j]->location[0];
                    diff_vector[1] = powerups[i]->location[1] - players[j]->location[1];
                    diff_vector[2] = powerups[i]->location[2] - players[j]->location[2];

                    sum += diff_vector[0] * diff_vector[0];
                    sum += diff_vector[1] * diff_vector[1];
                    sum += diff_vector[2] * diff_vector[2];
                    sum = sqrt(sum);
            
                    /* hit - whats a player's radius?
                     *
                     */
                    if ( sum < powerups[i]->radius ) { //players[j]->radius  ) {
                        int k;
                        double rad = 3.0;//players[j]->radius;

#ifdef SOUND_EFFECTS
                        // snd_sfx_play(0, 240, 1 );
                        snd_sfx_play( 11, 240, 1 );
#endif


                        switch( powerups[i]->type ) {
                            
                        case 0:
                            players[j]->hull_integrity += 75;
                            if ( players[j]->hull_integrity > MAX_HULL_INTEGRITY )
                                players[j]->hull_integrity = MAX_HULL_INTEGRITY;
                        break;

                        case 1:
                            players[j]->machine_gun = 1;
                            break;

                        case 2:
                            players[j]->spread_gun = 1;
                            break;

                        default: break;
                        }


                        /*** kill the powerup ***/
                        free( powerups[i] );
                        for ( k=i; k<powerup_count; k++ ) {
                            powerups[k] = powerups[k+1];
                        }
                        powerup_count--;


                    } /** hit **/



            } /** living **/
        }
    }




}









void gravity() {
    int j;
    for ( j=0; j<asteroid_count; j++ ) {
        if ( asteroids[j]->location[1] > 0.0 ) {
            asteroids[j]->location[1] -= 0.1;
        }

    }

}
