/* dcasteroids.h
   dreamcast asteroids v0.0.6: ogg bgm


 */
#include <kos.h>
#include <GL/gl.h>
#include <GL/glu.h>

#include <math.h>
#include <stdlib.h>

#include <oggvorbis/sndoggvorbis.h>


#define MAX_WALLS 100
#define MAX_TARGETS 100
#define MAX_SHOTS 400
#define MAX_ASTEROIDS 100
#define MAX_ENEMIES 2


#define GL_MODELVIEW_MATRIX 0

#define RUNSTATE_PLAYING 0
#define RUNSTATE_PAUSE   1
#define RUNSTATE_DONE    2

#define MAX_LASER_CHARGE 150
#define MAX_HULL_INTEGRITY 100
#define MAX_SHOT_POTENCY 150
#define TRIGGER_BRAKE_MAXIMUM 180
#define MIN_EXPLOSION_LIFETIME 10
#define PEAK_EXPLOSION_LIFETIME 30
#define MAX_EXPLOSION_LIFETIME 30

//#define MAX_ENEMY_LIFETIME 500
#define ENEMY_FLYAWAYTIME 200

#define MAX_RANDOM_PERCENTAGE 10000


/* compile options
 *
 *
 */
//#define TRANSP
#define SOUND_EFFECTS
//#define OGGVORBIS_BGM
//#define FRAME_RATE_COUNTER
//#define DCASTEROIDS_USE_PVR_SPHERES
#define RANDOM_UFOS
#define RANDOM_POWERUPS
//#define SHIP_MODEL_CODE
#define VERTICAL_LOOK_BOUND

/*** Dan Potter's code in macro form ***/
/* This weird logic is to avoid bouncing back
   and forth before the user lets go of the
   button. */
#define PRESSED( x, y ) if (!(cond.buttons & x )) { \
	if (!(y & 0x1000)) {

#define PRESSED2( x, y, z ) if (  (!(cond.buttons & x)) && \
          (!(cond.buttons & y)) ) { if (!(z & 0x1000)) {

#define DESSERP( y ) if (y == 0) y = 0x1001; else y = 0x1000; } }




/* v0.0.6: a subscreen 
 *  contains all the information a lpayer's display will need
 *
 */
typedef struct {
    int x;
    int y;
    int width;
    int height;
    int scoretext_x;
    int scoretext_y;
    int messagetext_x;
    int messagetext_y;
    
} subscreen_t;



/*** player structure ***/
typedef struct {

    double location[4];
    double velocity[4];
    double rotation[4];
    double direction[4];
    double topwards[4];
    double pitch, yaw, roll;
  
    int zoom;
    int chosen_view;
    int laser_charge;

    int damage_hits;
    int hull_integrity;
    int score;
    int shots_fired;
    int shots_hit;
    

    int braking; //state - a button
    int pausing;
    int living;
    int machine_gun;
    int spread_gun;
    
    int chosen_option;

    /*** v0.0.7: color to distinguish ***/
    float color[4];

    /*** v0.0.4: user preferences ***/
    int sensitivity_tight;
    int sensitivity_wide;
    int fov_tight;
    int fov_wide;

    /*** these are for Dan Potter's button-hold logic ***/
    int transa;
    int transb;
    int transx;
    int transy;
    int transltrig;
    int transrtrig;
    int transrtrigdeep;
    int transstart;
    int transview;
    int last_ltrig;
    subscreen_t *screen;
} shooter_t;

/*** v0.0.6: enemy structure ***/
typedef struct {
    double location[4];
    double velocity[4];
    double direction[4];
    int hull_integrity;
    int shot_cooldown;
    int lifetime;
    shooter_t *opponent;
} enemy_t;



/*** a shot ***/
typedef struct {
    double origin[4];
    double direction[4];
    int potent;
    shooter_t *owner;

} shot_t;


/*** a shot ***/
typedef struct {
    double location[4];
    double direction[4];
    int lifetime;
    shooter_t *owner;
} moving_shot_t;

/*** an asteroid ***/
typedef struct {
    double location[4];
    double velocity[4];
    double rotation[4];
    double rotation_speed[4];
    int size;
    double radius;
} asteroid_t;

/*** a shot ***/
typedef struct {
    double location[4];
    int lifetime;
    double radius;
} explosion_t;



/*** a powerup ***/
typedef struct {
    double location[4];
    double velocity[4];
    double rotation[4];
    double rotation_speed[4];
    int type;
    double radius;
} powerup_t;
